// Copyright (C) 2009 Playedit.

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "config.h"

#include "mtp_device.h"
#include "basic_dialogs.h"
#include "playlist.h"

#include <glibmm/i18n.h>
#include <iostream>

// Set a cell editable and start editing.
void PlaylistView::SetPlaylistNameEditable()
{
	Glib::RefPtr<Gtk::TreeSelection> selection = get_selection();
	if (selection->count_selected_rows())
	{
		m_PlaylistName.property_editable() = true;
		Gtk::TreePath path = std::vector<Gtk::TreePath>(selection->get_selected_rows()).front();
		Gtk::TreeViewColumn *column = get_column(0);
		set_cursor(path, *column, m_PlaylistName, true);
	}
}

// Delete selected playlists from the device.
void PlaylistView::DeletePlaylists()
{
	if (ConfirmationDialog(_("Delete Playlists"), _("Permanently delete playlists?")))
	{
		std::vector<Gtk::TreePath> selected_rows = get_selection()->get_selected_rows();
		std::vector<Gtk::TreeRowReference> row_refs = ConvertPathsToRowRefs(selected_rows);
		for (std::vector<Gtk::TreeRowReference>::iterator ref = row_refs.begin();
				ref != row_refs.end(); ++ref)
		{
			Gtk::TreeIter iter = m_Store->get_iter(ref->get_path());
			Glib::RefPtr<playlist_t> playlist =
				Glib::RefPtr<playlist_t>::cast_static(iter->get_value(m_StoreColumns.m_Data));
			// Remove the playlist from the device and the store
			if (m_Device.DeleteObject(playlist->Id()) &&
					(iter = m_Store->erase(iter)))
				get_selection()->select(iter);
		}
	}
}

// Find the correct position for a new playlist
static int FindNewPlaylistPosition(const Gtk::TreeNodeChildren& playlist_list,
		const Glib::RefPtr<playlist_t>& playlist,
		const Gtk::TreeModelColumn< Glib::RefPtr<mtp_data_t> >& column)
{
	int new_pos = 0;
	Gtk::TreeIter iter = playlist_list.begin();
	while (iter != playlist_list.end() &&
			PlaylistIsLess(Glib::RefPtr<playlist_t>::cast_static(iter->get_value(column)),
				playlist))
	{
		++iter;
		++new_pos;
	}
	return new_pos;
}

// Rename selected playlist on device.
void PlaylistView::RenamePlaylist(const Glib::ustring& string_path, const Glib::ustring& new_name)
{
	if (new_name.length())
	{
		Gtk::TreeIter iter = m_Store->get_iter(string_path);
		Glib::RefPtr<playlist_t> playlist =
			Glib::RefPtr<playlist_t>::cast_static(iter->get_value(m_StoreColumns.m_Data));

		// Change the playlist file name on the device
		m_Device.SetPlaylistName(playlist, new_name);

		// Find the new position for the playlist
		int count = FindNewPlaylistPosition(m_Store->children(), playlist, m_StoreColumns.m_Data);

		// Get the current position of the playlist in the playlist store
		int index = Gtk::TreePath(string_path)[0];

		// Move the playlist into its correct position in the playlist store
		if (index != count)
		{
			if (index < count) ++count;
			m_Store->move(iter, m_Store->children()[count]);
		}
	}
	m_PlaylistName.property_editable() = false;
}

void PlaylistView::CancelRenamePlaylist()
{
	m_PlaylistName.property_editable() = false;
}

// Create a new playlist on the device.
void PlaylistView::NewPlaylist()
{
	Glib::ustring newname =
		EntryDialog::RunEntryDialog(_("New Playlist"), _("Enter name for new playlist:"));
	if (newname.length())
	{
		Glib::RefPtr<playlist_t> playlist = playlist_t::create();
		playlist->SetName(newname);

		if (m_Device.CreateNewPlaylist(playlist))
		{
			// Get the playlist name the device actually used (may be
			// different due to device naming restrictions)
			playlist = m_Device.GetPlaylist(playlist->Id());

			// Find the position for the new playlist
			int count = FindNewPlaylistPosition(m_Store->children(), playlist, m_StoreColumns.m_Data);

			// Put the playlist into the store and select it
			Gtk::TreeIter iter = m_Store->insert(m_Store->children()[count]);
			(*iter)[m_StoreColumns.m_Data] = playlist;
			get_selection()->unselect_all();
			get_selection()->select(iter);
		}
	}
}

// Remove selected tracks from a playlist.
void PlaylistView::RemoveTracksFromPlaylist()
{
	if (ConfirmationDialog(_("Remove Tracks"), _("Remove selected tracks from the playlist?")))
	{
		std::vector<Gtk::TreePath> selected_paths = get_selection()->get_selected_rows();
		Gtk::TreePath parent_path(selected_paths.front());
		if (!parent_path.up())
			return;
		Gtk::TreeIter parent = m_Store->get_iter(parent_path);

		Glib::RefPtr<playlist_t> playlist =
			Glib::RefPtr<playlist_t>::cast_static(parent->get_value(m_StoreColumns.m_Data));

		std::vector<uint32_t> new_tracks = playlist->Tracks();

		for (std::vector<Gtk::TreePath>::reverse_iterator cur_selection = selected_paths.rbegin();
				cur_selection != selected_paths.rend(); ++cur_selection)
			new_tracks.erase(new_tracks.begin() + (*cur_selection)[1]);

		playlist->SetTracks(new_tracks);

		if (m_Device.UpdatePlaylist(playlist))
		{
			// Convert the selected paths to row references
			std::vector<Gtk::TreeRowReference> refs = ConvertPathsToRowRefs(selected_paths);
			for (std::vector<Gtk::TreeRowReference>::iterator ref = refs.begin();
					ref != refs.end(); ++ref)
				m_Store->erase(m_Store->get_iter(ref->get_path()));
		}
		else
			// Update didn't work.  Reset the tracks
			playlist->SetTracks(m_Device.GetPlaylist(playlist->Id())->Tracks());
	}
}
